@@ -1,22 +1,48 @@
-#!/usr/bin/env python2
-
-'''
-'''
+# ----------------------------------------------------------------------------
+# pyglet
+# Copyright (c) 2006-2008 Alex Holkner
+# Copyright (c) 2008-2019 pyglet contributors
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+#  * Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+#  * Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in
+#    the documentation and/or other materials provided with the
+#    distribution.
+#  * Neither the name of pyglet nor the names of its
+#    contributors may be used to endorse or promote products
+#    derived from this software without specific prior written
+#    permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+# ----------------------------------------------------------------------------
 from __future__ import absolute_import
 
-__docformat__ = 'restructuredtext'
-__version__ = '$Id: $'
-
 from ctypes import *
 
-from pyglet.canvas.xlib import XlibCanvas
 from .base import Config, CanvasConfig, Context
-
-from pyglet import gl
+from pyglet.canvas.xlib import XlibCanvas
 from pyglet.gl import glx
 from pyglet.gl import glxext_arb
 from pyglet.gl import glx_info
 from pyglet.gl import glxext_mesa
+from pyglet import gl
 
 
 class XlibConfig(Config):
@@ -206,15 +232,16 @@ class BaseXlibContext(Context):
 
         self._have_SGI_video_sync = config.glx_info.have_extension('GLX_SGI_video_sync')
         self._have_SGI_swap_control = config.glx_info.have_extension('GLX_SGI_swap_control')
+        self._have_EXT_swap_control = config.glx_info.have_extension('GLX_EXT_swap_control')
         self._have_MESA_swap_control = config.glx_info.have_extension('GLX_MESA_swap_control')
 
         # In order of preference:
-        # 1. GLX_MESA_swap_control (more likely to work where video_sync will
-        #    not)
-        # 2. GLX_SGI_video_sync (does not work on Intel 945GM, but that has
-        #    MESA)
-        # 3. GLX_SGI_swap_control (cannot be disabled once enabled).
-        self._use_video_sync = (self._have_SGI_video_sync and not self._have_MESA_swap_control)
+        # 1. GLX_EXT_swap_control (more likely to work where video_sync will not)
+        # 2. GLX_MESA_swap_control (same as above, but supported by MESA drivers)
+        # 3. GLX_SGI_video_sync (does not work on Intel 945GM, but that has EXT)
+        # 4. GLX_SGI_swap_control (cannot be disabled once enabled)
+        self._use_video_sync = (self._have_SGI_video_sync and
+                                not (self._have_EXT_swap_control or self._have_MESA_swap_control))
 
         # XXX mandate that vsync defaults on across all platforms.
         self._vsync = True
@@ -225,7 +252,9 @@ class BaseXlibContext(Context):
     def set_vsync(self, vsync=True):
         self._vsync = vsync
         interval = vsync and 1 or 0
-        if not self._use_video_sync and self._have_MESA_swap_control:
+        if not self._use_video_sync and self._have_EXT_swap_control:
+            glxext_arb.glXSwapIntervalEXT(self.x_display, glx.glXGetCurrentDrawable(), interval)
+        elif not self._use_video_sync and self._have_MESA_swap_control:
             glxext_mesa.glXSwapIntervalMESA(interval)
         elif self._have_SGI_swap_control:
             glxext_arb.glXSwapIntervalSGI(interval)
@@ -245,7 +274,7 @@ class XlibContext10(BaseXlibContext):
         super(XlibContext10, self).__init__(config, share)
 
     def _create_glx_context(self, share):
-        if self.config._requires_gl_3():
+        if self.config.requires_gl_3():
             raise gl.ContextException(
                 'Require GLX_ARB_create_context extension to create ' +
                 'OpenGL 3 contexts.')
@@ -298,7 +327,7 @@ class XlibContext13(BaseXlibContext):
         self.glx_window = None
 
     def _create_glx_context(self, share):
-        if self.config._requires_gl_3():
+        if self.config.requires_gl_3():
             raise gl.ContextException(
                 'Require GLX_ARB_create_context extension to create ' +
                 'OpenGL 3 contexts.')
@@ -313,7 +342,7 @@ class XlibContext13(BaseXlibContext):
                                        True)
 
     def attach(self, canvas):
-        if canvas is self.canvas:  # XXX do this for carbon too?
+        if canvas is self.canvas:
             return
 
         super(XlibContext13, self).attach(canvas)
