@@ -1,15 +1,16 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
+# Copyright (c) 2008-2019 pyglet contributors
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions 
+# modification, are permitted provided that the following conditions
 # are met:
 #
 #  * Redistributions of source code must retain the above copyright
 #    notice, this list of conditions and the following disclaimer.
-#  * Redistributions in binary form must reproduce the above copyright 
+#  * Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
 #    the documentation and/or other materials provided with the
 #    distribution.
@@ -33,7 +34,7 @@
 # ----------------------------------------------------------------------------
 # $Id: $
 
-'''Text formatting, layout and display.
+"""Text formatting, layout and display.
 
 This module provides classes for loading styled documents from text files,
 HTML files and a pyglet-specific markup format.  Documents can be styled with
@@ -70,8 +71,9 @@ Refer to the Programming Guide for advanced usage of the document and layout
 classes, including interactive editing, embedding objects within documents and
 creating scrollable layouts.
 
-:since: pyglet 1.1
-'''
+.. versionadded:: 1.1
+"""
+from builtins import object
 
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
@@ -81,16 +83,18 @@ import os.path
 import pyglet
 from pyglet.text import layout, document, caret
 
+
 class DocumentDecodeException(Exception):
-    '''An error occurred decoding document text.'''
+    """An error occurred decoding document text."""
     pass
 
+
 class DocumentDecoder(object):
-    '''Abstract document decoder.
-    '''
+    """Abstract document decoder.
+    """
 
     def decode(self, text, location=None):
-        '''Decode document text.
+        """Decode document text.
         
         :Parameters:
             `text` : str
@@ -100,11 +104,12 @@ class DocumentDecoder(object):
                 referenced within the document (for example, HTML images).
 
         :rtype: `AbstractDocument`
-        '''
+        """
         raise NotImplementedError('abstract')
 
+
 def get_decoder(filename, mimetype=None):
-    '''Get a document decoder for the given filename and MIME type.
+    """Get a document decoder for the given filename and MIME type.
 
     If `mimetype` is omitted it is guessed from the filename extension.
 
@@ -128,7 +133,7 @@ def get_decoder(filename, mimetype=None):
             filename.
 
     :rtype: `DocumentDecoder`
-    '''
+    """
     if mimetype is None:
         _, ext = os.path.splitext(filename)
         if ext.lower() in ('.htm', '.html', '.xhtml'):
@@ -147,9 +152,10 @@ def get_decoder(filename, mimetype=None):
         return attributed.AttributedTextDecoder()
     else:
         raise DocumentDecodeException('Unknown format "%s"' % mimetype)
-    
+
+
 def load(filename, file=None, mimetype=None):
-    '''Load a document from a file.
+    """Load a document from a file.
 
     :Parameters:
         `filename` : str
@@ -163,15 +169,24 @@ def load(filename, file=None, mimetype=None):
             supported MIME types.
 
     :rtype: `AbstractDocument`
-    '''
+    """
     decoder = get_decoder(filename, mimetype)
-    if file is None:
-        file = open(filename)
+    if not file:
+        with open(filename) as f:
+            file_contents = f.read()
+    else:
+        file_contents = file.read()
+        file.close()
+
+    if hasattr(file_contents, "decode"):
+        file_contents = file_contents.decode()
+
     location = pyglet.resource.FileLocation(os.path.dirname(filename))
-    return decoder.decode(file.read(), location)
+    return decoder.decode(file_contents, location)
+
 
 def decode_html(text, location=None):
-    '''Create a document directly from some HTML formatted text.
+    """Create a document directly from some HTML formatted text.
 
     :Parameters:
         `text` : str
@@ -181,12 +196,13 @@ def decode_html(text, location=None):
             referenced from the document (e.g., images).
 
     :rtype: `FormattedDocument`
-    '''
+    """
     decoder = get_decoder(None, 'text/html')
     return decoder.decode(text, location)
 
+
 def decode_attributed(text):
-    '''Create a document directly from some attributed text.
+    """Create a document directly from some attributed text.
 
     See `pyglet.text.formats.attributed` for a description of attributed text.
 
@@ -195,33 +211,36 @@ def decode_attributed(text):
             Attributed text to decode.
 
     :rtype: `FormattedDocument`
-    '''
+    """
     decoder = get_decoder(None, 'text/vnd.pyglet-attributed')
     return decoder.decode(text)
 
+
 def decode_text(text):
-    '''Create a document directly from some plain text.
+    """Create a document directly from some plain text.
 
     :Parameters:
         `text` : str
             Plain text to initialise the document with.
 
     :rtype: `UnformattedDocument`
-    '''
+    """
     decoder = get_decoder(None, 'text/plain')
     return decoder.decode(text)
 
+
 class DocumentLabel(layout.TextLayout):
-    '''Base label class.
+    """Base label class.
 
     A label is a layout that exposes convenience methods for manipulating the
     associated document.
-    '''
+    """
+
     def __init__(self, document=None,
                  x=0, y=0, width=None, height=None,
-                 anchor_x='left', anchor_y='baseline', 
+                 anchor_x='left', anchor_y='baseline',
                  multiline=False, dpi=None, batch=None, group=None):
-        '''Create a label for a given document.
+        """Create a label for a given document.
 
         :Parameters:
             `document` : `AbstractDocument`
@@ -245,15 +264,15 @@ class DocumentLabel(layout.TextLayout):
                 characters.  You must also set the width of the label.
             `dpi` : float
                 Resolution of the fonts in this layout.  Defaults to 96.
-            `batch` : `Batch`
+            `batch` : `~pyglet.graphics.Batch`
                 Optional graphics batch to add the label to.
-            `group` : `Group`
+            `group` : `~pyglet.graphics.Group`
                 Optional graphics group to use.
 
-        '''
-        super(DocumentLabel, self).__init__(document, 
-                                            width=width, height=height, 
-                                            multiline=multiline, 
+        """
+        super(DocumentLabel, self).__init__(document,
+                                            width=width, height=height,
+                                            multiline=multiline,
                                             dpi=dpi, batch=batch, group=group)
 
         self._x = x
@@ -262,90 +281,90 @@ class DocumentLabel(layout.TextLayout):
         self._anchor_y = anchor_y
         self._update()
 
-    def _get_text(self):
+    @property
+    def text(self):
+        """The text of the label.
+
+        :type: str
+        """
         return self.document.text
 
-    def _set_text(self, text):
+    @text.setter
+    def text(self, text):
         self.document.text = text
 
-    text = property(_get_text, _set_text,
-                    doc='''The text of the label.
-                    
-    :type: str
-    ''')
+    @property
+    def color(self):
+        """Text color.
 
-    def _get_color(self):
+        Color is a 4-tuple of RGBA components, each in range [0, 255].
+
+        :type: (int, int, int, int)
+        """
         return self.document.get_style('color')
 
-    def _set_color(self, color):
-        self.document.set_style(0, len(self.document.text), 
+    @color.setter
+    def color(self, color):
+        self.document.set_style(0, len(self.document.text),
                                 {'color': color})
 
-    color = property(_get_color, _set_color,
-                     doc='''Text color.
+    @property
+    def font_name(self):
+        """Font family name.
 
-    Color is a 4-tuple of RGBA components, each in range [0, 255].
+        The font name, as passed to :py:func:`pyglet.font.load`.  A list of names can
+        optionally be given: the first matching font will be used.
 
-    :type: (int, int, int, int)
-    ''')
-
-    def _get_font_name(self):
+        :type: str or list
+        """
         return self.document.get_style('font_name')
 
-    def _set_font_name(self, font_name):
-        self.document.set_style(0, len(self.document.text), 
+    @font_name.setter
+    def font_name(self, font_name):
+        self.document.set_style(0, len(self.document.text),
                                 {'font_name': font_name})
 
-    font_name = property(_get_font_name, _set_font_name,
-                         doc='''Font family name.
+    @property
+    def font_size(self):
+        """Font size, in points.
 
-    The font name, as passed to `pyglet.font.load`.  A list of names can
-    optionally be given: the first matching font will be used.
-
-    :type: str or list
-    ''')
-
-    def _get_font_size(self):
+        :type: float
+        """
         return self.document.get_style('font_size')
 
-    def _set_font_size(self, font_size):
-        self.document.set_style(0, len(self.document.text), 
+    @font_size.setter
+    def font_size(self, font_size):
+        self.document.set_style(0, len(self.document.text),
                                 {'font_size': font_size})
 
-    font_size = property(_get_font_size, _set_font_size,
-                         doc='''Font size, in points.
+    @property
+    def bold(self):
+        """Bold font style.
 
-    :type: float
-    ''') 
-
-    def _get_bold(self):
+        :type: bool
+        """
         return self.document.get_style('bold')
 
-    def _set_bold(self, bold):
-        self.document.set_style(0, len(self.document.text), 
+    @bold.setter
+    def bold(self, bold):
+        self.document.set_style(0, len(self.document.text),
                                 {'bold': bold})
 
-    bold = property(_get_bold, _set_bold,
-                    doc='''Bold font style.
+    @property
+    def italic(self):
+        """Italic font style.
 
-    :type: bool
-    ''')
-
-    def _get_italic(self):
+        :type: bool
+        """
         return self.document.get_style('italic')
 
-    def _set_italic(self, italic):
-        self.document.set_style(0, len(self.document.text), 
+    @italic.setter
+    def italic(self, italic):
+        self.document.set_style(0, len(self.document.text),
                                 {'italic': italic})
 
-    italic = property(_get_italic, _set_italic,
-                      doc='''Italic font style.
-                      
-    :type: bool
-    ''')
-
     def get_style(self, name):
-        '''Get a document style value by name.
+        """Get a document style value by name.
 
         If the document has more than one value of the named style,
         `pyglet.text.document.STYLE_INDETERMINATE` is returned.
@@ -356,11 +375,11 @@ class DocumentLabel(layout.TextLayout):
                 `pyglet.text.layout` for known style names.
 
         :rtype: object
-        '''
+        """
         return self.document.get_style_range(name, 0, len(self.document.text))
 
     def set_style(self, name, value):
-        '''Set a document style value by name over the whole document.
+        """Set a document style value by name over the whole document.
 
         :Parameters:
             `name` : str
@@ -369,20 +388,22 @@ class DocumentLabel(layout.TextLayout):
             `value` : object
                 Value of the style.
 
-        '''
+        """
         self.document.set_style(0, len(self.document.text), {name: value})
 
+
 class Label(DocumentLabel):
-    '''Plain text label.
-    '''
-    def __init__(self, text='', 
+    """Plain text label.
+    """
+
+    def __init__(self, text='',
                  font_name=None, font_size=None, bold=False, italic=False,
                  color=(255, 255, 255, 255),
-                 x=0, y=0, width=None, height=None, 
+                 x=0, y=0, width=None, height=None,
                  anchor_x='left', anchor_y='baseline',
                  align='left',
                  multiline=False, dpi=None, batch=None, group=None):
-        '''Create a plain text label.
+        """Create a plain text label.
 
         :Parameters:
             `text` : str
@@ -421,14 +442,14 @@ class Label(DocumentLabel):
                 characters.  You must also set the width of the label.
             `dpi` : float
                 Resolution of the fonts in this layout.  Defaults to 96.
-            `batch` : `Batch`
+            `batch` : `~pyglet.graphics.Batch`
                 Optional graphics batch to add the label to.
-            `group` : `Group`
+            `group` : `~pyglet.graphics.Group`
                 Optional graphics group to use.
 
-        '''
+        """
         document = decode_text(text)
-        super(Label, self).__init__(document, x, y, width, height, 
+        super(Label, self).__init__(document, x, y, width, height,
                                     anchor_x, anchor_y,
                                     multiline, dpi, batch, group)
 
@@ -441,17 +462,19 @@ class Label(DocumentLabel):
             'align': align,
         })
 
+
 class HTMLLabel(DocumentLabel):
-    '''HTML formatted text label.
+    """HTML formatted text label.
     
     A subset of HTML 4.01 is supported.  See `pyglet.text.formats.html` for
     details.
-    '''
-    def __init__(self, text='', location=None, 
+    """
+
+    def __init__(self, text='', location=None,
                  x=0, y=0, width=None, height=None,
                  anchor_x='left', anchor_y='baseline',
                  multiline=False, dpi=None, batch=None, group=None):
-        '''Create a label with an HTML string.
+        """Create a label with an HTML string.
 
         :Parameters:
             `text` : str
@@ -478,28 +501,28 @@ class HTMLLabel(DocumentLabel):
                 and line breaks.  You must also set the width of the label.
             `dpi` : float
                 Resolution of the fonts in this layout.  Defaults to 96.
-            `batch` : `Batch`
+            `batch` : `~pyglet.graphics.Batch`
                 Optional graphics batch to add the label to.
-            `group` : `Group`
+            `group` : `~pyglet.graphics.Group`
                 Optional graphics group to use.
 
-        ''' 
+        """
         self._text = text
         self._location = location
         document = decode_html(text, location)
-        super(HTMLLabel, self).__init__(document, x, y, width, height, 
+        super(HTMLLabel, self).__init__(document, x, y, width, height,
                                         anchor_x, anchor_y,
                                         multiline, dpi, batch, group)
 
-    def _set_text(self, text):
-        self._text = text
-        self.document = decode_html(text, self._location)
+    @property
+    def text(self):
+        """HTML formatted text of the label.
 
-    def _get_text(self):
+        :type: str
+        """
         return self._text
 
-    text = property(_get_text, _set_text, 
-                    doc='''HTML formatted text of the label.
-
-    :type: str
-    ''')
+    @text.setter
+    def text(self, text):
+        self._text = text
+        self.document = decode_html(text, self._location)
